from bedrockz._imports import *
import typer
import questionary
from bedrockz import core as cr
from bedrockz import files as flz
from bedrockz import config as cfg
from anyio import run
app = typer.Typer()
items_app = typer.Typer()
app.add_typer(items_app, name="items")


linker = typer.Typer()
app.add_typer(linker, name="linker")




@linker.command("list")
def list_projects_pm():
    typer.echo(f"Selling item:")

async def _link_project(path: Path):
    async with cfg.get_config(path) as box:
        packages = flz.BrowniePM()
        bcopy = box.copy()
        prior_remappings = bcopy.get("solidity", {}).get("remappings", [])
        box.solidity.remappings = list(set(packages.remappings + prior_remappings))
        # merged = cr.automerge(box.copy(), bcopy)
        # logger.info(merged)
        # box.update(merged)
        
        logger.warning(box)
        

@linker.command("attach")
def link_project():
    is_curr: bool = questionary.confirm("Would you like to link the current project?").ask()
    if not is_curr:
        typer.secho("Not Linking the current working directory.", fg="red", bold=True)
        return 
    
    root = cfg.vscode_config()
    run(_link_project, root)


@linker.command("detach")
def unlink_project(user_name: str):
    typer.echo(f"Deleting user: {user_name}")


if __name__ == "__main__":
    app()



from bedrockz._imports import *
from jinja2 import Template, Environment, FileSystemLoader
from yapf.yapflib.yapf_api import FormatCode



def get_environment(path: Path) -> Environment:
    """Get the environment for a given path."""
    loader = FileSystemLoader(path)
    return Environment(loader=loader)

class TemplateRenderer:
    def __init__(self, path: Path) -> None:
        self.env = get_environment(path)
    
    @property
    def templates(self) -> list:
        """Get all templates in a path."""
        return self.env.list_templates()
    
    def __setattr__(self, __name: str, __value: Any) -> None:
        if hasattr(self, "env"):
            logger.debug(f"Setting {__name} to {__value} inside of the environment too.")
            self.env.globals[__name] = __value    
        self.__dict__[__name] = __value
    
    
    def render(self, template_name: str, context: dict = {}) -> str:
        """Render a template with a context."""
        template = self.env.get_template(template_name)
        return template.render(context)
    
    def render_python(self, template_name: str, context: dict = {}) -> str:
        """Render a template with a context."""
        unformatted = self.render(template_name, context)
        formatted_code, _ = FormatCode(unformatted)
        return formatted_code
    


def main():
    curr = Path(__file__).parent
    local_temp_folder = curr / "templates"
    renderer = TemplateRenderer(local_temp_folder)
    # logger.success(list(Path(local_temp_folder).glob("*")))
    logger.success(renderer.templates)
    renderer.decimal_count = 18
    renderer.token_name = "Coinye Coin"
    renderer.token_symbol = "CYC"
    complete = renderer.render("brownie/token.py.jinja2")
    expandsive = f"\n\n{complete}"
    
    logger.success(expandsive)
if __name__ == "__main__":
    main()


import uuid
from typing import (Any, Callable, Dict, Iterable, List, Optional, Tuple,
                    TypeVar, Union)

from inflection import camelize, parameterize, tableize, titleize, underscore



    


def snake_case(name: str) -> str:
    return underscore(name.strip().replace(" ", "_")).lower()

def lowercase(name: str) -> str:
    return name.strip().replace(" ", "").lower()



from bedrockz._imports import *


def _type_match(type_tuple: Tuple[Any, Any]) -> bool:
    
    if len(type_tuple) < 2:
        return True
    
    t1, t2 = type_tuple
    return (
            type(t1) == type(t2)
            or isinstance(t2, type(t1)) 
            or isinstance(t1, type(t2))
            )

def merge_fn(val_list: List[Any]):
    list_set = orjson.loads(orjson.dumps(val_list))
    if not val_list:
        raise ValueError("No values to compare")
    type_list = pipe(val_list, 
                     partition_all(2), 
                     map(_type_match), 
                     filter(lambda x: not bool(x)), 
                     filter(lambda x: x is not None), 
                     list)
    
    
    if not type_list:
        if isinstance(list_set[0], list):
            return concat(list_set)
        if isinstance(list_set[0], dict):
            return merge(*list_set)
        if all(list_set[0] == val for val in list_set):
            return list_set[0]
        return list(val_list)
    
    grouped = pipe(val_list, groupby(lambda x: type(x).name), valmap(lambda x: x[0] if isinstance(x[0], list) else x))


    return grouped


def convert_dict(box_base: Box):
    jsonified = box_base.to_json()
    resp = orjson.loads(jsonified)
    

    return resp
            
def automerge(
    dict1: Union[Box, BoxList, Dict],
    dict2: Union[Box, BoxList, Dict],
) -> Dict[str, Any]:
    """Append a value to a dict."""
    compare_box = None
    merge_box = None
    
    if isinstance(dict1, (Box)):
        compare_box = convert_dict(dict1)
    if isinstance(dict2, (Box)):
        merge_box = convert_dict(dict2)

    merged = merge_with(merge_fn, compare_box, merge_box)
    return merged



# def 


# def name():
#     pass

# import addict
# from loguru import logger
# from matplotlib.pyplot import title
# import networkx as nx
# from addict import Addict as DDict
# from auto_all import end_all
# from auto_all import start_all
# from toolz import curry
# from mako.lookup import TemplateLookup
# from pydantic import BaseModel
# from pydantic import root_validator
# from pydantic import validate_arguments
# from sqlalchemy.engine.base import Engine
# from sqlalchemy.sql import ClauseElement
# from retworkx.visualization import graphviz_draw
# from contextlib import contextmanager

# _T = TypeVar("_T")


# FOLDER_PATH = Path(__file__).parent
# MAKO_LOOPUP = TemplateLookup(
#     directories=[(FOLDER_PATH / "templates"), (FOLDER_PATH / "service")],
#     output_encoding="utf-8",
#     encoding_errors="replace",
# )

# start_all(globals())


# def singleton(cls):
#     instance = [None]

#     def wrapper(*args, **kwargs):
#         if instance[0] is None:
#             instance[0] = cls(*args, **kwargs)
#         return instance[0]

#     return wrapper


# @singleton
# class DynamicGlobalMap(addict.Dict):
#     pass


# def consistent_naming(name: str) -> str:
#     return underscore(parameterize(name))


# def consistent_classes(name: str) -> str:
#     return camelize(underscore(name))


# def consistent_table(name: str) -> str:
#     return titleize(parameterize(name))


# @validate_arguments
# def int_sect(a: set, b: set) -> set:
#     """Get the intersection of a and b sets.

#     Args:
#         a (set): Set we're getting from.
#         b (set): Set we're trying to exclude.

#     Returns:
#         set: [description]
#     """
#     return set(a) & set(b)


# @validate_arguments
# def get_common(a: set, b: set) -> set:
#     """Get the intersection of a and b sets.

#     Args:
#         a (set): Set we're getting from.
#         b (set): Set we're trying to exclude.

#     Returns:
#         set: [description]
#     """
#     return set(a) & set(b)


# @validate_arguments
# def sym_diff(a: set, b: set) -> set:
#     return a.symmetric_difference(b)


# @curry
# def in_set(posts: Iterable[Any], item: Any) -> bool:
#     return item in posts


# @lru_cache
# def make_cache(node_json: str):
#     return hashlib.md5(node_json).hexdigest()  # type: ignore


# # ----------------------------------------------------------------
# #                        SQL Utils
# # ----------------------------------------------------------------

# # from sqlalchemy.sql.expression import Vis


# @curry
# def execute(statement: ClauseElement, engine: Engine):
#     return engine.execute(statement)


# # ----------------------------------------------------------------
# #                        NetworkX Shit
# # ----------------------------------------------------------------


# def bfs_preds(G: nx.Graph, source, depth_limit=None):
#     return nx.bfs_predecessors(G, source=source, depth_limit=depth_limit)


# def template_by_name(name: str):
#     return MAKO_LOOPUP.get_template(name)


# import orjson
# import xxhash


# class hashabledict(dict):
#     def __hash__(self):
#         x = xxhash.xxh32()
#         x.update(
#             orjson.dumps(self, option=orjson.OPT_NON_STR_KEYS | orjson.OPT_SORT_KEYS)
#         )
#         return x.intdigest()


# class InsertParameters(BaseModel):
#     data: Dict[str, Any]
#     event_at: datetime = datetime.now()
#     bucket: Optional[str]
#     tags: Dict[str, Any] = {}

#     @root_validator
#     def check_bucket(cls, values: dict):
#         doti = DDict(**values)
#         buck = doti.bucket
#         if buck:
#             doti.bucket = tableize(buck)
#             return doti.to_dict()

#         if "bucket" not in doti.data.keys():
#             raise ValueError("Bucket not added.")

#         nested_bucket = doti.data.bucket
#         if not isinstance(nested_bucket, str):
#             raise TypeError(
#                 "A bucket was found in 'data' but it wasn't the right type."
#             )
#         doti.data.pop("bucket", None)

#         doti.bucket = str(nested_bucket)
#         return doti.to_dict()


# def hexid() -> str:
#     return uuid.uuid4().hex[:5]


# def create_token():
#     pass


# def __dataclass_transform__(
#     *,
#     eq_default: bool = True,
#     order_default: bool = False,
#     kw_only_default: bool = False,
#     field_descriptors: Tuple[Union[type, Callable[..., Any]], ...] = (()),
# ) -> Callable[[_T], _T]:
#     return lambda a: a


# def graphviz_show(
#     graph,
#     node_attr_fn=None,
#     edge_attr_fn=None,
#     graph_attr=None,
#     filename: Optional[str] = None,
#     image_type=None,
#     method=None,
# ):
#     drawing = graphviz_draw(
#         graph, node_attr_fn, edge_attr_fn, graph_attr, method=method
#     )

#     if drawing is None:
#         logger.info(
#             "There was some error with drawing the graph. Empty response returned."
#         )

#         return
#     drawing.show(title=(filename.split(".")[0] if filename else "A Graph Image"))


# def is_list(value):
#     return isinstance(value, list)


# def listify(value: Union[Any, List[Any]]) -> List[Any]:
#     if not is_list(value):
#         return [value]
#     return value


# def first_key(d: dict) -> Union[str, int]:
#     return next(iter(d))


# end_all(globals())



from bedrockz._imports import *
from bedrockz.core import merge_fn, automerge
from bedrockz.files import get_root, vscode_config, app_folder


def attach_list(incoming: Union[BoxList, Box], value: Any) -> Union[BoxList , List]:
    """Append a value to a dict."""
    if isinstance(incoming, (BoxList, list)):
        incoming.append(incoming)
        return incoming
    
    return [value]


def attach_dict(incoming: Union[BoxList, Box], value: Dict[str, Any]) -> Union[BoxList , List]:
    """Append a value to a dict."""
    if not isinstance(incoming, (Box, Dict)):
        
        incoming.append(incoming)
        return incoming
    
    return [value]


async def to_box(value: Union[str, bytes]) -> dict:
    """Get a default dict."""
    value = {} if not value else orjson.loads(value) # type: ignore
    
    return Box(value, default_box=True)


@asynccontextmanager
async def get_config(path: Path):
    apath = AioPath(path)
    json_box = Box({}, default_box=True)
    try:
        file_str = await apath.read_bytes()
        json_box = await to_box(file_str) # type: ignore
        yield json_box

    finally:
        await apath.write_text(orjson.dumps(json_box.to_dict()).decode("utf-8"))
    


async def main():
    async with get_config(vscode_config()) as bdd:
        b1 = bdd.copy()
        b2 = bdd.copy()
        b1.hello = [1, 2]
        b2.hello = [3, 4]
        # merge_fn([b1, b2])
        
        b3 = bdd.copy()
        b4 = bdd.copy()
        
        b3.hello = [1, 2]
        b4.hello = ""
        # merged = merge_fn([b3, b4])
        merged = automerge(b3, b4)
        logger.info(merged)
        

if __name__ == "__main__":
    run(main)





# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['bedrockz', 'bedrockz.generator']

package_data = \
{'': ['*'], 'bedrockz.generator': ['templates/brownie/*']}

install_requires = \
['Jinja2>=3.1.1,<4.0.0',
 'addict>=2.4.0,<3.0.0',
 'anyio[all]>=3.5.0,<4.0.0',
 'auto-all>=1.4.1,<2.0.0',
 'blue>=0.8.0,<0.9.0',
 'cytoolz>=0.11.2,<0.12.0',
 'diskcache>=5.4.0,<6.0.0',
 'forbiddenfruit>=0.1.4,<0.2.0',
 'fs>=2.4.15,<3.0.0',
 'inflection>=0.5.1,<0.6.0',
 'loguru>=0.6.0,<0.7.0',
 'matplotlib>=3.5.1,<4.0.0',
 'maya>=0.6.1,<0.7.0',
 'networkx>=2.8,<3.0',
 'orjson>=3.6.7,<4.0.0',
 'pydantic>=1.9.0,<2.0.0',
 'python-box[msgpack,toml,ruamel.yaml]==6.0.0',
 'questionary>=1.10.0,<2.0.0',
 'retworkx[all]>=0.11.0,<0.12.0',
 'typer[all]>=0.4.1,<0.5.0',
 'xxhash>=3.0.0,<4.0.0',
 'yapf>=0.32.0,<0.33.0']

entry_points = \
{'console_scripts': ['brockz = bedrockz.cli:app']}

setup_kwargs = {
    'name': 'bedrockz',
    'version': '0.2.0',
    'description': '',
    'long_description': None,
    'author': 'Kevin H.',
    'author_email': 'kevin@autoworkz.org',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



# isort: off

from auto_all import start_all, end_all

start_all(globals())
# isort: on
import itertools
import uuid
from contextlib import asynccontextmanager
from pathlib import Path
from typing import (Any, Callable, Dict, Iterable, List, Optional, Tuple,
                    TypeVar, Union)

import fs
import orjson
import typer
from anyio import Path as AioPath
from anyio import run
from box import Box, BoxList
from cytoolz.curried import (concat, filter, groupby, itemfilter, itemmap,
                             keyfilter, keymap, map, merge, merge_with,
                             partition_all, pipe, unique, valfilter, valmap)
from forbiddenfruit import curse
from fs import open_fs
from inflection import camelize, parameterize, tableize, titleize, underscore
from loguru import logger
from packaging import version

# isort: off



def type_name(self):
    return self.__name__

curse(type, "name", property(type_name))

end_all(globals())

# isort: on



from bedrockz._imports import *

from bedrockz import format as fmt

IMPORTANT_PATHS = {
    "BROWNIE": Path.home() / ".brownie" / "packages",
    "DEFAULT_ROOT_FILES": ["pyproject.toml", "pyproject.lock", "requirements.txt", "setup.py"],
}


def get_dir(path: Path) -> Path:
    if path.is_dir():
        return path
    return path.parent

def get_cwd(file: Optional[str] = None) -> Path:
    """Get current working directory or passed in file."""
    return Path.cwd() if (not file) else get_dir(Path(file))


def get_root(start: Optional[Path] = None, by_files:list=[]) -> Path:
    """Get root directory of a project by looking for a file in the directory tree."""
    count = 0
    start = get_cwd() if (not start) else start
    current = start
    
    while count < 3:
        opened_fs = open_fs(start.as_posix())
        iterator = list(opened_fs.walk.files(filter=by_files))
        
        if iterator:
            
            relative_file = iterator[0][1:]
            current = (current / relative_file).parent
            break
        count +=1
        current = start.parent
    return current

def get_contract_folders(root: Path) -> list:
    """Get all contract folders in a project."""
    sol_files = get_contract_files(root)
    
    folders = pipe(sol_files, 
                   map(lambda x: get_parent_above("contracts", x)), 
                   unique, 
                   list)
    return folders

def get_parent_above(name: str, path: Path) -> Path:
    """Recursively look for the level above a parent folder with a given name."""
    return path.parent if (name == path.parts[-1]) else get_parent_above(name, path.parent)
    
def get_contract_files(root: Path) -> Iterable[Path]:
    """Get all contract files in a project."""
    sol_files = root.glob("**/contracts/**/*.sol")
    return sol_files

def force_exist(path: Path) -> Path:
    if not path.exists():
        if not path.is_dir():
            path.parent.mkdir(parents=True)
            path.touch(exist_ok=True)
            return path
        path.mkdir(parents=True)
    return path

"""
# Specific Smart Contract Commands Here
---

"""
def get_brownie_pms():
    """Get all brownie packages providers"""
    brownie_packages = get_contract_folders(IMPORTANT_PATHS["BROWNIE"])
    provider_packages: Dict[str, List[Path]] = {}
    for package in brownie_packages:
        parent_name = package.parent.name
        if parent_name not in provider_packages:
            provider_packages[parent_name] = []
        provider_packages[parent_name].append(package)
        # provider_packages[pacekage.parent] = get_contract_files(package)
    return provider_packages




# VSCode File Manipulation

def vscode_config(root: Optional[Path] = None, force: bool=True) -> Path:
    """Get the vscode settings file."""
    root_files = IMPORTANT_PATHS["DEFAULT_ROOT_FILES"]
    root = get_root(by_files=root_files) if (not root) else root # type: ignore
    return force_exist(root / ".vscode" / "settings.json")


def app_folder(name: str, author: str="curious", version: str="0.0.1"):
    return open_fs(f'usercache://{name}:{author}:{version}')


class BrowniePM:
    def __init__(self) -> None:
        self.pkg_set: Dict[str, List[Path]] = keymap(fmt.lowercase, get_brownie_pms())
    
    @property
    def root(self):
        return IMPORTANT_PATHS["BROWNIE"]
    
    
    
    @property
    def groups(self) -> List[str]:
        return list(self.pkg_set.keys())

    @property
    def remapped_dict(self) -> Dict[str, Path]:
        primaries = {}
        for group, packages in self.pkg_set.items():
            primaries[f"@{group}"] = list(sorted(packages, key=lambda x: version.parse(x.name.split("@")[1])))[0]
        return primaries
    @property
    def remappings(self) -> List[str]:
        _remappings = []
        for lib, package in self.remapped_dict.items():
            if lib.startswith("@"):
                _remappings.append(f"{lib}={package.as_posix()}")
                continue
            _remappings.append(f"@{lib}={package.as_posix()}")
        return _remappings
    
# class CacheFS:
#     def __init__(self, name: str):
#         self.name = name
#         self.app_dir = typer.get_app_dir(name)
    
#     def __enter__(self):
        
#         return self
    
#     def __exit__(self, exc_type, exc_val, exc_tb):
#         self    
# class CacheFS:
#     def __init__(self, name: str):
#         self.name = name
#         self.app_dir = typer.get_app_dir(name)
    
#     def __enter__(self):
        
#         return self
    
#     def __exit__(self, exc_type, exc_val, exc_tb):
#         self
     

def main():
    brownie_pms = get_brownie_pms()
    # get_contract_folders(IMPORTANT_PATHS["BROWNIE"])
    map(lambda x: x.relative_to(), brownie_pms)
    logger.info(brownie_pms)

if __name__ == "__main__":
    main()
